from ylz_utils.langchain.loaders.docx_loader import DocxLoader
from ylz_utils.langchain.loaders.image_loader import ImageLoader
from ylz_utils.langchain.loaders.pptx_loader import PptxLoader
from ylz_utils.langchain.loaders.url_loader import UrlLoader


class LoaderLib():
   def __init__(self):
      self.url = UrlLoader()
      self.docx = DocxLoader()
      self.pptx = PptxLoader()
      self.image = ImageLoader()


from langchain_community.vectorstores import FAISS

class FaissLib():
    def __init__(self,langchainLib):
        self.langchainLib = langchainLib

    def create_from_docs(self,docs,embedding=None) -> FAISS:
        if not embedding:
            embedding = self.langchainLib.get_embedding()
        vectorstore = FAISS.from_documents(docs,embedding=embedding)
        return vectorstore
    def create_from_textes(self,textes,embedding=None) -> FAISS:
        if not embedding:
            embedding = self.langchainLib.get_embedding()
        vectorstore = FAISS.from_texts(textes, embedding=embedding)
        return vectorstore
       
    def save(self,  db_file:str, vectorstore: FAISS,index_name:str = "index"):
        vectorstore.save_local(db_file,index_name)

    def load(self, db_file:str ,embedding=None, index_name:str = "index") -> FAISS:
        if not embedding:
            embedding = self.langchainLib.get_embedding()
        vectorestore = FAISS.load_local(db_file, embeddings=embedding, index_name=index_name, allow_dangerous_deserialization=True)
        return vectorestore
    
    def search(self,query,vectorstore: FAISS,k=10):
        return vectorstore.similarity_search(query,k=k)



from langchain_community.document_loaders import RecursiveUrlLoader

class UrlLoader():
    def loader(self, url, max_depth=2, extractor=None, metadata_extractor=None):
        #"./example_data/fake.docx"
        loader = RecursiveUrlLoader(
            url = url,
            max_depth= max_depth,
            # use_async=False,
            extractor= extractor,
            metadata_extractor= metadata_extractor
            # exclude_dirs=(),
            # timeout=10,
            # check_response_status=True,
            # continue_on_failure=True,
            # prevent_outside=True,
            # base_url=None,
        )
        return loader.load()


#from langchain_community.document_loaders import UnstructuredPowerPointLoader
from pptx import Presentation

class PptxLoader():
    def loader(self, filename: str):
        #"./example_data/fake-power-point.pptx"
        ppt = Presentation(filename)
        for slide in ppt.slides:
            for shape in slide.shapes:
                if shape.has_text_frame:
                    text_frame = shape.text_frame
                    print(text_frame.text)
